/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file ppedset.hpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _PpedSet_HPP
#define _PpedSet_HPP

#include "dynset/ppedsetintf.h"

namespace capd{
namespace dynset{

template<typename MatrixType>
typename PpedSet<MatrixType>::ScalarType PpedSet<MatrixType>::size(void)
{
   return ::size(m_x + m_B*m_r);
}

template<typename MatrixType>
void PpedSet<MatrixType>::move(capd::dynsys::DynSys<MatrixType>& dynsys)
{
   VectorType s(m_x.dimension());
   MatrixType R(m_x.dimension(),m_x.dimension());

   VectorType xx = m_x + m_B*m_r;

   VectorType y = dynsys.Phi(m_x);
   split(y,s);
   s += dynsys.Remainder(xx);
   m_B = dynsys.JacPhi(xx)*m_B;
   splitMatrix(m_B,R);
   m_x = y;
   VectorType rr = gauss(m_B,R*m_r+s);

   m_r += rr;
}

template<typename MatrixType>
void PpedSet<MatrixType>::move(capd::dynsys::DynSys<MatrixType>& dynsys, PpedSet& result) const
{
   VectorType s(m_x.dimension());
   MatrixType R(m_x.dimension(),m_x.dimension());

   VectorType xx = m_x + m_B*m_r;

   VectorType y = dynsys.Phi(m_x);
   split(y,s);
   s += dynsys.Remainder(xx);
   result.m_B = dynsys.JacPhi(xx)*m_B;
   splitMatrix(result.m_B,R);
   result.m_x = y;
   VectorType rr = gauss(result.m_B,R*m_r+s);

   result.m_r = m_r+rr;
}

template<typename MatrixType>
std::string PpedSet<MatrixType>::show(void) const
{
   std::ostringstream descriptor;
   descriptor << "PpedSet: x=";
   descriptor << m_x << "\n B=";
   descriptor << m_B << "\n r=";
   descriptor << m_r << " ";
   return descriptor.str();
}

template<typename MatrixType>
typename PpedSet<MatrixType>::VectorType PpedSet<MatrixType>::affineTransformation(
      const MatrixType& A_M, const VectorType& A_C) const
{
   return A_M*(m_x-A_C) + (A_M*m_B)*m_r;
}

}} // namespace capd::dynset

#endif

/// @}
