/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file rectset.hpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _RectSet_HPP
#define _RectSet_HPP

#include "dynset/rectsetintf.h"

namespace capd{
namespace dynset{

template<typename MatrixType>
typename RectSet<MatrixType>::ScalarType RectSet<MatrixType>::size(void)
{
   return ::size(m_x + m_B*m_r);
}

template<typename MatrixType>
void RectSet<MatrixType>::move(capd::dynsys::DynSys<MatrixType>& dynsys)
{
   VectorType s(m_x.dimension());
   MatrixType C(m_x.dimension(),m_x.dimension());

   VectorType xx = m_x + m_B*m_r;

   VectorType y = dynsys.Phi(m_x);
   split(y,s);
   s += dynsys.Remainder(xx);
   MatrixType D = dynsys.JacPhi(xx)*m_B;
   MatrixType Q = midMatrix(D);
   orthonormalize(Q);
   MatrixType Qtr = Transpose(Q);
   m_x = y;
   m_r = (Qtr*D)*m_r + Qtr*s;
   m_B = Q;
}

template<typename MatrixType>
void RectSet<MatrixType>::move(capd::dynsys::DynSys<MatrixType>& dynsys, RectSet<MatrixType>& result) const
{
   VectorType s(m_x.dimension());
   MatrixType C(m_x.dimension(),m_x.dimension());

   VectorType xx = m_x + m_B*m_r;

   VectorType y = dynsys.Phi(m_x);
   split(y,s);
   s += dynsys.Remainder(xx);
   MatrixType D = dynsys.JacPhi(xx)*m_B;
   MatrixType Q = midMatrix(D);
   orthonormalize(Q);
   MatrixType Qtr = Transpose(Q);
   result.m_x = y;
   result.m_r = (Qtr*D)*m_r + Qtr*s;
   result.m_B = Q;
}

template<typename MatrixType>
std::string RectSet<MatrixType>::show(void) const
{
   std::ostringstream descriptor;
   descriptor << "RectSet: x=";
   descriptor << m_x << "\n B=";
   descriptor << m_B << "\n r=";
   descriptor << m_r << " ";
   return descriptor.str();
}

template<typename MatrixType>
typename RectSet<MatrixType>::VectorType RectSet<MatrixType>::affineTransformation(
      const MatrixType& A_M, const VectorType& A_C) const
{
   return A_M*(m_x-A_C) + (A_M*m_B)*m_r;
}

}} // namespace capd::dynset

#endif // _RectSet_HPP

/// @}
