/// @addtogroup dynsys
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file basicC2TaylorIntf.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/** Author: Daniel Wilczak, 2003-2005 */

#ifndef _BasicC2TaylIntf_H
#define _BasicC2TaylIntf_H

#include <string>
#include "vectalg/normintf.h"
#include "map/mapintf.h"
#include "map/c2coeffintf.h"
#include "dynsys/basicTaylorIntf.h"

namespace capd{
namespace dynsys{

template<typename MapT>
class BasicC2Taylor : public virtual BasicTaylor<MapT>{
public:
   typedef MapT MapType;
   typedef typename MapT::FunctionType FunctionType;
   typedef typename MapType::MatrixType MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename MatrixType::ScalarType ScalarType;
   typedef Norm<VectorType,MatrixType> NormType;
   typedef typename MapType::C2CoeffType C2CoeffType;

   BasicC2Taylor(const MapType& vectorField, int Order, const ScalarType& Step);
   BasicC2Taylor(const std::string& vectorField, int Order, const ScalarType& Step);
   BasicC2Taylor(const char* vectorField, int Order, const ScalarType& Step);
   BasicC2Taylor(const BasicC2Taylor &);
   ~BasicC2Taylor();
   
   BasicC2Taylor& operator=(const BasicC2Taylor&);
   VectorType operator()(const VectorType&, MatrixType&, C2CoeffType&);

   // If someone knows how to use 'using BasicTaylor<MapType>::operator()' in the borland compiler?:(
   VectorType operator()(const VectorType& v) const
   {
      return BasicTaylor<MapType>::operator()(v);
   }
   VectorType operator()(const VectorType& v, MatrixType& der) const
   {
      return BasicTaylor<MapType>::operator()(v,der);
   }
   void setField(const MapType& newVectorField);
   void setOrder(int newOrder);

   using BasicTaylor<MapType>::getField;
   using BasicTaylor<MapType>::getOrder;
   using BasicTaylor<MapType>::getStep;
   using BasicTaylor<MapType>::setStep;
   using BasicTaylor<MapType>::setParameter;
   using BasicTaylor<MapType>::getCoeff;
   using BasicTaylor<MapType>::getMatrixCoeff;

protected:
   using BasicTaylor<MapType>::computeMatrixCoeff;
   using BasicTaylor<MapType>::m_vField;
   using BasicTaylor<MapType>::m_order;
   using BasicTaylor<MapType>::m_step;
   using BasicTaylor<MapType>::m_coeff;
   using BasicTaylor<MapType>::m_matrixCoeff;
   using BasicTaylor<MapType>::m_dimension;
   using BasicTaylor<MapType>::m_F;

   C2CoeffType* m_c2MatrixCoeff;
   void computeC2Coeff(const VectorType& x, const MatrixType& V, const C2CoeffType& H, int ile) const;
}; // the end of class BasicC2Taylor


// ------------------ inline definitions -------------

template <typename MapType>
inline BasicC2Taylor<MapType>::~BasicC2Taylor()
{
   delete [] m_c2MatrixCoeff;
}

}} // the end of the namespace capd::dynsys

#endif // _BasicC2TaylIntf_H

/// @}
