/// @addtogroup dynsys
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file c2dynsys.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _C2DynSys_H_
#define _C2DynSys_H_

#include <string>
#include "dynsys/c1dynsys.h"
#include "map/c2coeff.h"

namespace capd{
namespace dynsys{

template<typename MatrixT>
class C2DynSys : public virtual capd::dynsys::C1DynSys<MatrixT>{
public:
   typedef MatrixT MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename MatrixType::ScalarType ScalarType;
   typedef Norm<VectorType,MatrixType> NormType;
   typedef capd::map::C2Coeff<ScalarType> C2CoeffType;

   virtual void c2Phi(const VectorType& x, MatrixType& jac, C2CoeffType& hessian) const = 0;
   virtual void c2Enclosure(
         const VectorType &W1,
         const MatrixType &W3,
         NormType &the_norm,
         C2CoeffType& result
   ) const = 0;
   virtual void c2Remainder(
         const VectorType& Enclosure,
         const MatrixType& jacEnclosure,
         const C2CoeffType& hessianEnclosure,
         VectorType& Remainder,
         MatrixType& jacRemainder,
         C2CoeffType& hessianRemainder
   ) const = 0;

   using C1DynSys<MatrixType>::JacRemainder;
   using C1DynSys<MatrixType>::jacEnclosure;
   using C1DynSys<MatrixType>::enclosure;

// from DynSys
   using C1DynSys<MatrixType>::Phi;
   using C1DynSys<MatrixType>::JacPhi;
   using C1DynSys<MatrixType>::Lipschitz;
   using C1DynSys<MatrixType>::type;
};

}} // namespace capd::dynsys

#endif

/// @}
