/// @addtogroup vectalg

/// @{



/////////////////////////////////////////////////////////////////////////////

/// @file subVector.hpp

///

/// @author Tomasz Kapela

/////////////////////////////////////////////////////////////////////////////



// Copyright (C) 2007 by the CAPD Group.

//

// This file is part of the CAPD library.  This library is free software;

// you can redistribute it and/or modify it under the terms of the GNU

// General Public License as published by the Free Software Foundation;

// either version 2 of the License, or (at your option) any later version.

//

// This library is distributed in the hope that it will be useful,

// but WITHOUT ANY WARRANTY; without even the implied warranty of

// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

// GNU General Public License for more details.

//

// You should have received a copy of the GNU General Public License along

// with this software; see the file "license.txt".  If not, write to the

// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,

// MA 02111-1307, USA.



#if !defined(__SUBVECTOR_HPP__)

#define __SUBVECTOR_HPP__





template<typename VectorType1, typename VectorType2>

void intersectionWithSubvector(const VectorType1 & src, VectorType2 & target, 

                               int start, int stop)

{

   bool result = true;

   for(int iTarget=start, iSrc=0; iTarget<stop; ++iTarget, ++iSrc)

      result = (result && intersection (src[iSrc], target[iTarget], target[iTarget]));

   if(!result)

     throw std::runtime_error(" Null intersection in intersectionWithSubvector ");

}





template<typename VectorType1, typename VectorType2>

void copyToSubvector(const VectorType1 & src, VectorType2 & target, 

                     int start, int stop)

{

   for(int iTarget=start, iSrc=0; iTarget<stop; ++iTarget, ++iSrc)

      target[iTarget] = src[iSrc];



}



template<typename VectorType1, typename VectorType2>

void copyFromSubvector(const VectorType1 & src, VectorType2 & target, 

                       int start, int stop)

{

   for(int iSrc=start, iTarget=0; iSrc<stop; ++iTarget, ++iSrc)

      target[iTarget] = src[iSrc];



}



template<typename MatrixType1, typename MatrixType2>

void intersectionWithSubmatrix( const MatrixType1 & src, 

                                MatrixType2 & target, 

                                int startRow, int stopRow, 

                                int startCol, int stopCol)

{

   bool result =  true;

   for(int iTarget=startRow, iSrc=0; iTarget<stopRow; ++iTarget, ++iSrc)

      for(int jTarget=startCol, jSrc=0; jTarget<stopCol; ++jTarget, ++jSrc)

         result = (result && intersection(src[iSrc][jSrc], target[iTarget][jTarget], target[iTarget][jTarget]));

   if(!result)

     throw std::runtime_error(" Null intersection in intersectionWithSubvector ");

}





template<typename MatrixType1, typename MatrixType2>

void copyToSubmatrix(const MatrixType1 & src, 

                     MatrixType2 & target, 

                     int startRow, int stopRow, 

                     int startCol, int stopCol)

{

   for(int iTarget=startRow, iSrc=0; iTarget<stopRow; ++iTarget, ++iSrc)

      for(int jTarget=startCol, jSrc=0; jTarget<stopCol; ++jTarget, ++jSrc)

         target[iTarget][jTarget] = src[iSrc][jSrc];

}





template<typename MatrixType1, typename MatrixType2>

void copyFromSubmatrix(const MatrixType1 & src, 

                       MatrixType2 & target, 

                       int startRow, int stopRow, 

                       int startCol, int stopCol)

{

   for(int iSrc=startRow, iTarget=0; iSrc<stopRow; ++iTarget, ++iSrc)

      for(int jSrc=startCol, jTarget=0; jSrc<stopCol; ++jTarget, ++jSrc)

         target[iTarget][jTarget] = src[iSrc][jSrc];

}



#endif

