/// @addtogroup homologicalAlgebra
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) Marian Mrozek 2005-2006
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.



#include <iostream>
#include <fstream>
#include <sstream>
#include <exception>
#include <stdexcept>
#include <new>
using namespace std;

#include "homologicalAlgebra/embeddingDim.h"
#include "auxil/ofstreamcout.h"
#include "auxil/Stopwatch.h"
#include "auxil/memSize.h"
#include "auxil/CRef.h"

extern ofstreamcout fcout;

#include "homologicalAlgebra/homologicalAlgebra.hpp"
#include "bitSet/EuclBitSet.hpp"
#include "auxil/handleUnexpectedTerminate.h"


typedef unsigned long int word;

ofstreamcout fcout;

int main(int argc,char **argv){
  set_unexpected(handle_unexpected);
  set_terminate(handle_terminate);

  // Check arguments
  if (argc < 3) {
    std::cout << "Usage: " << argv[0] << " inputFile outputFile " << std::endl;
    std::cout << "       inputFile - a text file in cub format to be converted" << std::endl;
    std::cout << "       outputFile - the resulting binary file (bmd extension recommended)" << std::endl;
    exit(0);
  }

  // Open files
  fstream cubfile(argv[1],ios::in);

  if(!cubfile.good()){
    std::cout << "Unable to open CUB file " << argv[1] << " for reading " << std::endl;
    exit(0);
  }

  try{
    BCubSet& cubSet = *new BCubSet(256);
    cubfile >> cubSet;
    unsigned int multidimType='D'*256+'B'; // for multidimensional full cubical sets
    cubSet.writeBmp(argv[2],multidimType);
  }catch(std::exception& e){
    cout << "Caught exception: " << e.what() << endl;
  }catch(std::string& s){
    cout << "Caught exception: " << s.c_str() << endl;
  }catch(const char* c){
    cout << "Caught exception: " << c << endl;
  }catch(...){
    cout << "Caught an unknown exception: " << endl;
  }
}

/// @}

