#include <iostream>
#include <fstream>
#include <sstream>
#include <exception>
#include <new>
using namespace std;

#include "multiEngHom/MultiEngHomT.h"
#include "auxil/handleUnexpectedTerminate.h"
#include "auxil/commandLineArgs.h"

int main(int argc,char** argv){
  set_unexpected(handle_unexpected);
  set_terminate(handle_terminate);
  fcout.open("chomp_raport.txt");
  fcout.turnOn();

  setupCommandLineArgs;
  declareCommandLineArg(std::string,inFile,"");
  declareCommandLineArg(std::string,engine,"MM_CR");
  declareCommandLineArg(int,verbose,1);

  std::pair<int,std::string> vc= getValueCountAndArg(std::string("1"));
  if(vc.first>1){
    fcout << "Only one file name required\n";
    exit(1);
  }else if(vc.first==1){
    inFile=vc.second;
  }else if(inFile==std::string("")){
    fcout << "\n --- This is ChomP basic software for Betti numbers --- \n\n";
    fcout << "Basic usage: \n\n";
    fcout << "      mchomp filename\n\n";
    fcout << "ChomP has several engines for computing Betti numbers. \n";
    fcout << "In some situations some engines may work faster than the preselected one. \n";
    fcout << "To specify a concrete engine use: \n\n";
    fcout << "      mchomp filename engine=xx\n\n";
    fcout << "where xx denotes one of the available engines : MM_ASLT,MM_CR,MM_AR,PP,PP_AS,BK,BK_LT\n";
    exit(0);
  }

  try{
    MultiEngineHomology<0> ch;
    const int dim=3;
    int d[dim];
    const int* dims=d;
    const char* bitmap=0;
    int betti[dim];
    // The CR reference is stored in cubSetCR to avoid premature destroying of the bitmap
    CRef<BCubSet> cubSetCR=readCubFile<CubSetT<EuclBitSetT<BitSetT<BitmapT<unsigned long int> >,dim> > >(inFile.c_str(),dims,bitmap);
    ch(dim,dims,bitmap,betti,engine,verbose);
    fcout.turnOn();
    for(int i=0;i<dim;++i) fcout << "Betti[" << i << "]=" << betti[i]  << std::endl;
  }catch(std::exception& e){
    fcout.turnOn();
    fcout << "Caught exception: " << e.what() << endl;
  }catch(std::string& s){
    fcout.turnOn();
    fcout << "Caught exception: " << s.c_str() << endl;
  }catch(const char* c){
    fcout.turnOn();
    fcout << "Caught exception: " << c << endl;
  }catch(...){
    fcout.turnOn();
    fcout << "Caught an unknown exception: " << endl;
  }

}
