/// @addtogroup map
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file node.cpp
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include "interval/doubleInterval.h"
#include "map/node.h"
using namespace capd::map;

template class Node<double>;
template class ConsNode<double>;
template class VarNode<double>;
template class SumNode<double>;
template class DifNode<double>;
template class MulNode<double>;
template class DivNode<double>;
template class SinNode<double>;
template class CosNode<double>;
template class ExpNode<double>;
template class LogNode<double>;
template class SqrtNode<double>;
template class PowNode<double>;

template class MulConsNode<double>;
template class DivConsNode<double>;

template class Node<interval>;
template class ConsNode<interval>;
template class VarNode<interval>;
template class SumNode<interval>;
template class DifNode<interval>;
template class MulNode<interval>;
template class DivNode<interval>;
template class SinNode<interval>;
template class CosNode<interval>;
template class ExpNode<interval>;
template class LogNode<interval>;
template class SqrtNode<interval>;
template class PowNode<interval>;

template class MulConsNode<interval>;
template class DivConsNode<interval>;

template<>
double& PowNode<double>::operator()(int i)
{
   if(i<=m_maxComputedDerivative)
      return value[i];
   m_maxComputedDerivative = i;

   (*left)(i);
   (*right)(i);

   double c=right->value[0];
   if (i==0)
      return value[i] = pow(left->value[i],c);
   if (c==1)
      return value[i] = left->value[i];

   double result=0.;

   if(left->value[0]==0)
   {
      double *temp1 = new double[i+1];
      double *temp2 = new double[i+1];

      int j,k,p;
      for(j=0;j<=i;++j)
         temp1[j]=left->value[j];

      for(k=1;k<c;++k)
      {
         for(p=0;p<=i;++p)
         {
            temp2[p]=0.;
            for(j=0;j<=p;++j)
               temp2[p] += left->value[j]*temp1[p-j];
         }
         for(p=0;p<=i;++p)
            temp1[p]=temp2[p];
      }

      result = temp1[i];
      delete []temp1;
      delete []temp2;
      return value[i] = result;
   }

   for(int j=0;j<i;++j)
      result+= (i*c-j*(c+1)) * value[j] * left->value[i-j];

   return value[i] = result / (i * left->value[0]);
}

/// @}
