/// @addtogroup setarth
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file lorenz.h
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _lorenz_h_
#define _lorenz_h_

#include "capd/capdlib.h"

class Lorenz : public Ode{
public:
	Lorenz(const interval& the_s, const interval& the_R, const interval& the_q);
	intervalVector f(const intervalVector &iv) const;
	intervalMatrix df(const intervalVector &iv) const;
	intervalMatrix d2f(int i,const intervalVector &iv) const;
	interval getR(void) const;
protected:
	interval s,R,q;
};

class LorenzEuler : public DynSys{
public:
	LorenzEuler(Lorenz &the_lorenz, const interval& the_h);
	intervalVector Phi(const intervalVector &iv) const;
	intervalMatrix JacPhi(const intervalVector &iv) const;
	intervalVector Remainder(const intervalVector &iv) const;
protected:
	Lorenz &lorenz;
	interval h;
};


//inline definitions

inline Lorenz::Lorenz(const interval& the_s, const interval& the_R, const interval& the_q)
	: s(the_s),R(the_R),q(the_q)
{}

inline LorenzEuler::LorenzEuler(Lorenz &the_lorenz, const interval& the_h)
	: lorenz(the_lorenz),h(the_h)
{}

inline intervalVector LorenzEuler::Remainder(const intervalVector &iv) const
{
	intervalVector result;
	result.clear();
	return result;
}

inline interval Lorenz::getR(void) const
{
	return R;
}

#endif

/// @}
