/// @addtogroup setarth
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file rossler.cpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include "vectalg/vectalglib.h"
#include "rossler.h"

#if (DIM == 3)

intervalVector Rossler::f(const intervalVector &x) const
{
   intervalVector result;
   result[0]=-(x[1]+x[2]);
   result[1]=x[0]+b*x[1];
   result[2]=b + x[2]*(x[0]-a);
   return result;
}

intervalMatrix Rossler::df(const intervalVector &x) const
{
   intervalMatrix result;
   result[0][0]=0;
   result[0][1]=-1;
   result[0][2]=-1;
   result[1][0]=1;
   result[1][1]=b;
   result[1][2]=0;
   result[2][0]=x[2];
   result[2][1]=0;
   result[2][2]=x[0]-a;
   return result;
}

intervalMatrix Rossler::d2f(int i,const intervalVector &x) const
{
   intervalMatrix result;
   if(i==2){
      result[0][2]=1;
      result[2][0]=1;
   }

   return result;
}

intervalVector RosslerEuler::Phi(const intervalVector &x) const
{
   return x+h*rossler.f(x);
}

intervalMatrix RosslerEuler::JacPhi(const intervalVector &x) const
{
   return intervalMatrix::Identity(3)+h*rossler.df(x);
}

#endif

/// @}
